package Prod::VCS51::Common;

sub startprocs_sys {
    my ($prod,$sys)=@_;
    my ($at,$cfg,$procs);
    $cfg = Obj::cfg();
    return adjust_ru_procs if(Cfg::opt("upgrade_nonkernelpkgs"));
    $at=$sys->prod("AT50");
    $at->verify_vxatd_start_sys($sys);
    $procs = Prod::startprocs_sys($prod, $sys);
    $procs = $prod->verify_procs_list_sys($sys,$procs,"start");
    if ($cfg->{vcs_allowcomms}) {
        if ((!EDR::file_sys($sys,"$prod->{vxfenmode}")) && (!EDR::file_sys($sys,"$prod->{vxfendg}"))) {
            $procs = EDR::arrdel($procs, "vxfen51");
        }
        return $procs;
    } else {
        $procs = EDR::arrdel($procs, "llt51");
        $procs = EDR::arrdel($procs, "gab51");
        $procs = EDR::arrdel($procs, "vxfen51");
        return $procs;
    }
}

sub upgrade_prestop_sys {
    my($maincf,$prod,$sys);
    ($prod,$sys) = @_;
    if(Cfg::opt('upgrade_kernelpkgs')){
        $prod->tsub("ru_prestop_sys",$sys);
        return 1;
    } elsif(Cfg::opt('upgrade_nonkernelpkgs')){
        if($sys->system1){
            $prod->tsub("upgrade_rolling_post_sys",$sys);
            EDR::create_flag("upgrade_rolling_post");
        } else {
            EDR::wait_for_flag("upgrade_rolling_post");
        }
    }

    unless ($sys->{vcs_upgradeable} && $prod->upgrade_gabtab_sys($sys)){
        if($sys->system1){
            EDR::create_flag('maincf_pre_upgrade_done');
        }
        return 0;
    }

    $prod->upgrade_saveconf_sys($sys) unless (Cfg::opt("rollback"));
    # set secure cluster attribute to each system
    $maincf=EDR::cmd_sys($sys, "_cmd_cat $prod->{maincf}");
    EDR::set_value_sys($sys, "secureclus",1) if ($maincf=~/SecureClus\s*=\s*1/);

    # following tasks only performed on first node
    if ($sys->system1) {
        #return 0 unless ($prod->check_obsolete_types_sys($sys));

        # freeze service group
        if (!Cfg::opt("rootpath") && !$prod->{upgrade_ignore_conf} && $sys->{prod_running}) {
            $prod->haconf_makerw();
            $prod->freeze_groups();
            $prod->haconf_dumpmakero();
        }

        EDR::create_flag("maincf_pre_upgrade_done");
    } else {
        EDR::wait_for_flag("maincf_pre_upgrade_done");
    }

    return 1;
}

1;

package Prod::SFHA51::Common;

sub startprocs_sys {
    my ($prod,$sys)=@_;
    my ($at,$cfg,$vcs,$vm,$ref_procs);
    return adjust_ru_procs if(Cfg::opt("upgrade_nonkernelpkgs"));
    $cfg =Obj::cfg();
    $at=$sys->prod("AT50");
    $at->verify_vxatd_start_sys($sys);
    $vm = Obj::prod("VM51", $prod->{padv});
    $vcs = $sys->prod("VCS51");
    $ref_procs = Prod::startprocs_sys($prod, $sys);
    $ref_procs = $vm->tsub("verify_procs_list_sys",$sys,$ref_procs,"start");
    $ref_procs = $vcs->tsub("verify_procs_list_sys",$sys,$ref_procs,"start");
    if (!$cfg->{vcs_allowcomms}) {
        $ref_procs = EDR::arrdel($ref_procs, "llt51", "gab51", "vxfen51");
    } elsif ((!EDR::file_sys($sys,"$vcs->{vxfenmode}")) && (!EDR::file_sys($sys,"$vcs->{vxfendg}"))) {
        $ref_procs = EDR::arrdel($ref_procs, "vxfen51");
    }
    return $ref_procs;
}

1;

package SunOS;

sub patch_version_sys {
    my ($padv,$sys,$patch,$iv,$op,$pn,$pv,$force_flag,$cmd,$rp58,$rootpath,$obsolete_return,$patch_vers,$out);
    ($padv,$sys,$patch,$force_flag)=(@_);
    ($pn,$pv)=($patch->{patchname},$patch->{patchvers});
    $patch_vers=$pn."-".$pv;
    $op=$padv->tsub("patch_obsoleted_sys", $sys, $patch) if ($pv);
    $obsolete_return=[qw(999)];
    return $obsolete_return if ($op);
    # $padv->tsub("patches_sys",$sys);
    if ($force_flag) {
        $rootpath="-R ".Cfg::opt("rootpath") if (Cfg::opt("rootpath"));
        $rp58=1 if (($rootpath) && ($sys->{local_platvers} eq "5.8"));
        $cmd = ($rp58) ? "_cmd_patchadd" : "_cmd_showrev";
        $out = EDR::cmd_sys($sys, "$cmd -p $rootpath | _cmd_grep $patch_vers");
        $out =~ /Patch:\s\d+-(\d+)/;
        $iv = $1;
    } else {
        $iv=$sys->{patchvers}{$pn};
    }
    return $iv;
}

package CPIC;

sub start_processes_serial {
    my($cpic,$msg,$prod,$proc,$proci,$sys,$web,$reboot_systems);
    $cpic=shift;
    $prod=$cpic->prod;
    $web=Obj::web();

    if (Cfg::opt(qw(upgrade patchupgrade))) {
        for $sys(@{$cpic->{systems}}) {
        # only support reusing configure when patchupgrade
            if ((Cfg::opt("upgrade") || $cpic->{reuse_config})
                && ($prod->can(upgrade_sub("configure_sys")))) {
                # if upgrade scenario or product already configured when patchupgrade
                $msg=Msg::new("Performing $prod->{abbr} upgrade configuration on $sys->{sys}", 30, 1344, "$prod->{abbr}", "$sys->{sys}");
                $web->display_left($msg);
                $prod->tsub(upgrade_sub("configure_sys"), $sys);
            } elsif ($prod->can('configure_sys')) {
                # which means product not configured when patchupgrade
                $msg=Msg::new("Performing $prod->{abbr} configuration on $sys->{sys}", 30, 1345, "$prod->{abbr}", "$sys->{sys}");
                $web->display_left($msg);
                $prod->tsub("configure_sys", $sys);
            }
            $web->display_right();
        }
    }

    if ((Cfg::opt("configure")) && ($prod->can('configure_sys'))) {
        for $sys(@{$cpic->{systems}}) {
            $msg=Msg::new("Performing $prod->{abbr} configuration on $sys->{sys}", 30, 1345, "$prod->{abbr}", "$sys->{sys}");
            $web->display_left($msg);
            $prod->tsub("configure_sys", $sys);
            $web->display_right();
        }
    }

    $prod->tsub("config_errors",$cpic) if ($cpic->sys_error);
    EDR::reset_errors_warnings();
    return "" if (Cfg::opt(qw(rootpath nostart)));

    unless ($cpic->{reboot}) {
        for $proci(@{$cpic->{startprocs}}) {
            $proc=Obj::proc($proci, $cpic->{padv});
            next if $proc->{donotstart};
            for $sys(@{$cpic->{systems}}) {
                next unless EDR::inarr($proci,@{$sys->{startprocs}});
                next if EDR::inarr($proci,@{$sys->{donotstartprocs}});
                $msg=Msg::new("Starting $proc->{proc} on $sys->{sys}", 30, 1346, "$proc->{proc}", "$sys->{sys}");
                $web->display_left($msg);
                if ($cpic->tsub("proc_start_sys", $sys, $proc)) {
                    proc_start_passed_sys($sys, $proc);
                    $web->display_right();
                } else {
                    # setting the reboot flag for start failure
                    $sys->{reboot}=1;
                    proc_start_failed_sys($sys, $proc);
                    $web->display_right("failed");
                }
            }
        }

        $reboot_systems=$cpic->systems_need_reboot();
        unless (@{$reboot_systems}) {
            if ($prod->can(upgrade_sub("poststart_sys"))) {
                for $sys(@{$cpic->{systems}}) {
                    $msg=Msg::new("Performing $prod->{abbr} poststart tasks on $sys->{sys}", 30, 1347, "$prod->{abbr}", "$sys->{sys}");
                    $web->display_left($msg);
                    $prod->tsub(upgrade_sub("poststart_sys"), $sys);
                    $web->display_right();
                }
            }
        }

        $prod->tsub("poststart_errors",$cpic) if ($cpic->sys_error);
        EDR::reset_errors_warnings();
    }
    return "";
}

sub start_processes_threaded {
    my(@reboot_sys,$cpic,$failmsg,$msg,$padv,$prod,$proc,$proci,$sys,$thr,$web,$reboot_systems,$logfileid,$edr,$rtn);
    $cpic=shift;
    $prod=$cpic->prod;
    $web=Obj::web();
    $thr=Obj::thr();
    $edr=Obj::edr();
    $thr->tsub("mq_create_threads", $cpic->nsystems, $cpic->{threadlimit});

    if (Cfg::opt(qw(upgrade patchupgrade rollback))) {
        if ((Cfg::opt("upgrade") || $cpic->{reuse_config})
            && ($prod->can(upgrade_sub("configure_sys")))) {
            if (Cfg::opt("rollback")) {
                $msg=Msg::new("Performing $prod->{abbr} configuration", 30, 1348, "$prod->{abbr}");
            } else {
                $msg=Msg::new("Performing $prod->{abbr} upgrade configuration", 30, 1349, "$prod->{abbr}");
            }
            $web->display_left($msg);
            $thr->tsub("mq_add_sub_allsys", "CPIC", "prod_upgrade_configure_sys",
                $cpic->{prod});
            $thr->tsub("mq_wait_for_completion");
            $thr->tsub("mq_read_rtns");
        } elsif ($prod->can('configure_sys')) {
            $msg=Msg::new("Performing $prod->{abbr} configuration", 30, 1348, "$prod->{abbr}");
            $web->display_left($msg);
            $thr->tsub("mq_add_sub_allsys", "CPIC", "prod_configure_sys",
                $cpic->{prod});
            $thr->tsub("mq_wait_for_completion");
            $thr->tsub("mq_read_rtns");
        }
        $web->display_right();
    }

    if ((Cfg::opt("configure")) && ($prod->can('configure_sys'))) {
        $msg=Msg::new("Performing $prod->{abbr} configuration", 30, 1348, "$prod->{abbr}");
        $web->display_left($msg);
        $thr->tsub("mq_add_sub_allsys", "CPIC", "prod_configure_sys",
            $cpic->{prod});
        $thr->tsub("mq_wait_for_completion");
        $thr->tsub("mq_read_rtns");
        $web->display_right();
    }

    $prod->tsub("config_errors",$cpic) if ($cpic->sys_error);
    return "" if (Cfg::opt(qw(rootpath nostart)));

    unless ($cpic->{reboot}) {
        for $proci(@{$cpic->{startprocs}}) {
            $proc=Obj::proc($proci, $cpic->{padv});
            next if $proc->{donotstart};
            $msg=Msg::new("Starting $proc->{proc}", 30, 1339, "$proc->{proc}");
            $web->display_left($msg);
            if (($#{$cpic->{systems}}>0) && ($proc->{multisystemserialstart})) {
                $logfileid=0;
                for $sys(@{$cpic->{systems}}) {
                    $logfileid++;
                    next unless EDR::inarr($proci,@{$sys->{startprocs}});
                    next if EDR::inarr($proci,@{$sys->{donotstartprocs}});
                    $edr->{logfileid}=$logfileid;
                    $rtn=$cpic->tsub("proc_start_sys", $sys, $proc);
                    $edr->{logfileid}=0;
                    if ($rtn) {
                        proc_start_passed_sys($sys, $proc);
                    } else {
                        # setting the reboot flag for start failure
                        $sys->{reboot}=1;
                        EDR::set_value_sys($sys,'reboot', 1);
                        proc_start_failed_sys($sys, $proc);
                    }
                }
            } else {
                $thr->tsub("mq_add_sub_allsys", "CPIC", "proc_start_sys",
                    $proci, 1);
                $thr->tsub("mq_wait_for_completion");
                $thr->tsub("mq_read_rtns", "01", "CPIC::proc_start_passed_sys",
                           "CPIC::proc_start_failed_sys", $proc, 1);
            }
            $web->display_right();
        }

        $reboot_systems=$cpic->systems_need_reboot();
        unless (@{$reboot_systems}) {
            if ($prod->can(upgrade_sub("poststart_sys"))) {
                $msg=Msg::new("Performing $prod->{abbr} poststart tasks", 30, 1350, "$prod->{abbr}");
                $web->display_left($msg);
                if (($#{$cpic->{systems}}>0) && ($prod->{multisystemserialpoststart})) {
                    $logfileid=0;
                    for $sys(@{$cpic->{systems}}) {
                        $logfileid++;
                        $edr->{logfileid}=$logfileid;
                        $prod->tsub(upgrade_sub("poststart_sys"), $sys);
                        $edr->{logfileid}=0;
                    }
                } else {
                    $thr->tsub("mq_add_sub_allsys", "CPIC", "prod_poststart_sys",
                               $cpic->{prod});
                    $thr->tsub("mq_wait_for_completion");
                    $thr->tsub("mq_read_rtns");
                }
                $web->display_right();
            }

            $prod->tsub("poststart_errors",$cpic) if ($cpic->sys_error);
        }
    }

    $thr->tsub("mq_join_threads");
}

package Rel::UXRT51SP1RP2::Common;

sub init_common {
    my ($rel,$edr);
    $rel = shift;
    $rel->{verify_media}=1; # comment out once release layout is stable
    $rel->{reli}="UXRT";
    $rel->{reltitle}=Msg::new("Storage Foundation and High Availability Solutions", 40, 1000)->{msg};
    $rel->{vers}="5.1.102.0";
    $rel->{nopackages}=1;
    # In basevers LxRT 5.1SP1PR2 and AxRT 5.1SP1PR1 included
    $rel->{basevers}=[ "5.1.120.0", "5.1.110.0", "5.1.101.0", "5.1.100.200", "5.1.100.100", "5.1.100.0" ]; # Decreasing order necessary
    $rel->{base_releases} = [ qw(UXRT51SP1RP1) ];
    $rel->{currentrelease} = 'UXRT51SP1RP2';
    $rel->{gavers}="5.1";
    $rel->{prior_rp_title}{UXRT51SP1RP1}="5.1 SP1RP1";
    $rel->{titlevers}="5.1 SP1RP2";
    $rel->{type}="RP";
    $rel->{rp}=1;
    $rel->{patchtype}="rppatches";
    $rel->{vos_upload_config_url}="https://vias.symantec.com/data/uploader/uxrt51upload.conf";

    # keep args alphabetized or well grouped so they can be easily found
    $rel->{args_task} = [ qw(patchupgrade precheck rollback postcheck) ];

    $rel->{args_opt} = [ qw( listpatches missingpkgs bait mpok nospacecheck vxunroot makeresponsefile
                             pkginfo serial upgradecv upgrade_kernelpkgs upgrade_nonkernelpkgs ignorepatchreqs version nostart) ];

    $rel->{args_def} = [ qw(browserid hostfile jumpstart keyfile
                             kickstart nim patchpath prod rootpath ) ];

    $rel->{args_undoc} = [ qw(debug demo mpok prod require repkgs uuid
                               nocleanup browserid upgradecv bait oem nooem
                               debug trace mpok osversion vxunroot
                               mediapath nospacecheck missingpkgs
                               startstoptest patchupgrade nostart) ];

    $rel->{prods}=[ qw(FS51 DMP51 VM51 VCS51 SF51 SFHA51 SFCFS51 SFCFSHA51 SFRAC51 SFCFSRAC51 SVS51 AT50) ];
    $rel->{pkgsetprods}=[ qw(SFCFSRAC51 SFRAC51 SVS51 SFCFSHA51 SFCFS51 SFHA51 SF51 VCS51 DMP51 AT50) ];
    $rel->{menuprods}=[ qw(FS51 DMP51 VM51 VCS51 SF51 SFHA51 SFCFS51 SFCFSHA51 SFRAC51 SFCFSRAC51 SVS51 AT50) ];
    $rel->{menuprods}=EDR::arrdel($rel->{prods}, qw(FS51 VM51));
    $rel->{upgradeprods} = [ qw(SFRAC51 SFCFSHA51 SFCFS51 SFHA51 SF51 VCS51 VM51 DMP51 FS51 AT50) ];
    # products which support -fencing option.
    $rel->{fencingprods} = [ qw(SFRAC51 SVS51 SFCFSHA51 SFCFS51 SFHA51 VCS51) ];
    # products which support -security and -addnode options.
    $rel->{secureprods} = [ qw(SFRAC51 SVS51 SFCFSRAC51 SFCFSHA51 SFCFS51 SFHA51 VCS51) ];
    $rel->{pkgs}=[ qw(VRTSvlic32 VRTSperl510 VRTSspt51) ];
    $rel->{minpkgs}=[ qw(VRTSvlic32 VRTSperl510)];
    $rel->{recpkgs}=[ qw(VRTSvlic32 VRTSperl510 VRTSspt51) ];
    $rel->{ru_unkernelpkgs}=[ qw(VRTSvcs51 VRTSvcsag51 VRTSvcsea51 VRTScavf51 ) ];
    $rel->{ru_prod}=[ qw(SFRAC51 SFCFSRAC51 SFCFSHA51 SFCFS51 SFHA51  VCS51 ) ];
    $rel->{ru_version}="5.1";


    $rel->{installonupgradepkgs}=[ qw(VRTSsfmh VRTSvcsea VRTScps
                                      VRTSdbed VRTSodm VRTSspt VRTSat) ];

    $rel->{obsolete_scripts} = [qw( installfs       uninstallfs
                                    installvm       uninstallvm
                                    installsfora    uninstallsfora
                                    installsfsyb    uninstallsfsyb
                                    installsfdb2    uninstallsfdb2)];
    $edr=Obj::edr();
    if(-d "$edr->{mediapath}/storage_foundation_basic") {
        $rel->{menuprods}=[ qw(SF51) ];
        $rel->{verify_media}=0; # do not check release layout
    }
    $rel->{oem}=1 if ((-f "$edr->{mediapath}/.oem") ||
        (-f "$edr->{mediapath}/../.oem") ||
        (EDR::inarr("-oem",@ARGV)));
    $rel->{oem}="" if (EDR::inarr("-nooem",@ARGV));

    $rel->{cross_upgradable_matrix}= {
        "SFRAC" => [],
        "SFCFSRAC" => [],
        "SVS" => [],
        "SFCFSHA" => [],
        "SFCFS" => ["SFCFSHA"],
        "SFHA" => [],
        "SF" => [],
        "VCS" => [],
        "VM" => ["SF"],
        "DMP" => ["SF"],
        "FS" => ["SF"],
        "AT" => ["ALL"],
    };
    $rel->{partial_upgradable_matrix}= {
        "SFRAC" => [],
        "SFCFSRAC" => [],
        "SVS" => [],
        "SFCFSHA" => ["SFCFS"],
        "SFCFS" => [],
        "SFHA" => ["SF","VCS","VM","FS"],
        "SF" => ["VM","FS"],
        "VCS" => [],
        "VM" => [],
        "DMP" => [],
        "FS" => [],
        "AT" => [],
    };
    $rel->{independent_prod_matrix} = {
        "SFRAC" => [],
        "SFCFSRAC" => [],
        "SVS" => [],
        "SFCFSHA" => [],
        "SFCFS" => [],
        "SFHA" => [],
        "SF" => ["VCS"],
        "VCS" => ["SF","VM","DMP","FS"],
        "VM" => ["VCS","FS"],
        "DMP" => ["VCS","FS"],
        "FS" => ["VCS","VM","DMP"],
        "AT" => [],
    };

}

package Patch::VRTSvxvm_5_1_132_0::Linux;

sub preinstall_sys {
    my ($patch,$sys)=@_;
    my $vm=$sys->prod("VM51");
    if ((Cfg::opt("nostart")) && ( ! EDR::file_sys($sys,$vm->{mkdbfile}))) {
        EDR::cmd_sys($sys,"_cmd_touch $vm->{mkdbfile} 2>/dev/null");
        $sys->{cpi_mkdbfile}=1;
        EDR::set_value_sys($sys, 'cpi_mkdbfile', 1);
    }
    return;
}

sub postinstall_sys {
    my ($patch,$sys)=@_;
    my $vm=$sys->prod("VM51");
    if ((Cfg::opt("nostart")) && ($sys->{cpi_mkdbfile}) && EDR::file_sys($sys,$vm->{mkdbfile})) {
        EDR::cmd_sys($sys,"_cmd_rmr $vm->{mkdbfile} 2>/dev/null");
    }
    return;
}

1;

